'cGLWindow.inc class file by Kent Sarikaya and Charles Pegge
'Last Modified:
'2011.10.22 KS
'13:56 07/10/2013 CP
'08:52 09/03/2017 CP

'Member variables have a "m" prefix for member
'Parameters/Arguements have an "a" prefix for arguement

#case capital
% NULL null
% LOWORD loword
% HIWORD hiword
include "../../inc/minwin.inc"

'Needed for GL Files Below
#define WINGDIAPI
#define APIENTRY

include "cTimerLowRes.inc"
includepath + "../../inc/glo2/"
library "opengl32.dll"
include once "gl/gl.h"
' include once "gl/glext.h"
library "glu32.dll"
include once "gl/glu.h"
include once "gl/wgl.inc"
library ""


class cGLWindow

    private
        'vars
        sys mFullScreen, mTop, mLeft, mWidth, mHeight, mPixelFormat, mBPP
        string mCaption, mClassName
        sys mMenuCommand
        sys mMouseX, mMouseY, mMouseButton, mKeydown
        sys mChangeSize, mIsActive, mIsRunning, mToDestroy
        sys mRenderFunc, mInputFunc
        sys mFOV
        double mNearDistance, mFarDistance

        static sys mKeys[256]

        sys  mhInstance, mhDC, mhRC, mhWnd, mFont
        RECT mWindowRect
        MSG  mMSG
        sys  mExStyle, mStyle
    
    public
        'methods
        method Constructor( sys aRenderProc=0, sys aInputProc=0 )
            mInputFunc  = aInputProc
            mRenderFunc = aRenderProc
            mFullScreen = 0
            mFOV = 45
            mNearDistance = 0.1
            mFarDistance  = 100000000000000
        end method


        method Destructor()
        end method



        method Create( sys aWidth   =  800, 
                       sys aHeight  =  600, 
                       sys aBits    =   32, 
                       sys aFullscreen = 0, 
                       string aCaption = "Simple GLWindow",
                       sys aLeft = 0, 
                       sys aTop  = 0, 
                       sys aIcon = 0, 
                       sys aMenu = 0 ) as sys

            mWidth     = aWidth
            mHeight    = aHeight
            mBPP       = aBits
            mLeft      = aLeft
            mTop       = aTop
            mCaption   = aCaption
            mClassName = "GLWindow"

            mWindowRect.left   = mLeft
            mWindowRect.right  = mWidth
            mWindowRect.top    = mTop
            mWindowRect.bottom = mHeight

            mFullScreen = aFullscreen

            mhInstance  = GetModuleHandle 0
            '
            'WndProc interface function
            '==========================
            '
            static thisObject, pwin
            '
            thisObject=@this
            pWin=@WindowProc#sys#sys#sys#sys
            '
            function WndProc( sys mhWnd, uMsg, wParam, lParam ) as sys callback
                return call pWin(thisObject,mhWnd, uMsg, wParam, lParam )
            end function
            '
            WNDCLASS winClass
            winClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC
            winClass.lpfnWndProc   = @WndProc
            winClass.cbClsExtra    = 0
            winClass.cbWndExtra    = 0
            winClass.hInstance     = mhInstance
            winClass.hIcon         = LoadIcon NULL, IDI_WINLOGO 
            winClass.hCursor       = LoadCursor NULL, IDC_ARROW 
            winClass.hbrBackground = NULL
            winClass.lpszMenuName  = NULL
            winClass.lpszClassName = *mClassName
            if not RegisterClass @winClass then
                'error
                MessageBox NULL, "RegisterClass", "ERROR", MB_OK
                return 0
            end if
            Remake()
        end method


        method Remake()
            if mFullScreen then
                mWidth = 800 : mHeight = 600
            
                DEVMODE sc
                sc.dmSize       = sizeof sc
                sc.dmPelsWidth  = mWidth
                sc.dmPelsHeight = mHeight
                sc.dmBitsPerPel = mBPP
                sc.dmFields = DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL
                if ChangeDisplaySettings( @sc, CDS_FULLSCREEN ) <> DISP_CHANGE_SUCCESSFUL then
                    mFullScreen = 0
                    if MessageBox ( NULL, "Fullscreen Not Supported. Use Windowed Mode?","ERROR", MB_YESNO | MB_ICONEXCLAMATION ) = IDYES  then
                        else
                            MessageBox NULL, "Program Closing", "ERROR", MB_OK | MB_ICONSTOP
                            return 0
                    end if
                end if
            end if

            if mFullScreen then
                mExStyle = WS_EX_APPWINDOW
                mStyle = WS_POPUP
                ShowCursor 0
            else
                mExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE
                mStyle = WS_OVERLAPPEDWINDOW
            end if
            
            mWindowRect <= 0, 0, mWidth, mHeight
            AdjustWindowRectEx @mWindowRect, mStyle, 0 , mExStyle
            
            mhWnd = CreateWindowEx(  mExStyle , 
                                    *mClassName, 
                                    *mCaption, 
                                    mStyle | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, 
                                    mLeft, 
                                    mTop, 
                                    mWindowRect.right  - mWindowRect.left, 
                                    mWindowRect.bottom - mWindowRect.top , 
                                    NULL,    'hparent
                                    NULL,    'hmenu
                                    mhInstance, 
                                    NULL )   'lparam
            if mhWnd = 0 then
                Destroy()
                'Error the window could not be created
                MessageBox NULL, "CreateWindowEx", "ERROR", MB_OK
                return 0
            end if

            static PIXELFORMATDESCRIPTOR pfd
            pfd.nSize           = sizeof PIXELFORMATDESCRIPTOR
            pfd.nVersion        = 1
            pfd.dwFlags         = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER
            pfd.iPixelType      = PFD_TYPE_RGBA
            pfd.cColorBits      = mBPP
            pfd.cRedBits        = 0
            pfd.cRedShift       = 0
            pfd.cGreenBits      = 0
            pfd.cGreenShift     = 0
            pfd.cBlueBits       = 0
            pfd.cBlueShift      = 0
            pfd.cAlphaBits      = 0
            pfd.cAlphaShift     = 0
            pfd.cAccumBits      = PFD_MAIN_PLANE
            pfd.cAccumRedBits   = 0
            pfd.cAccumGreenBits = 0
            pfd.cAccumBlueBits  = 0
            pfd.cAccumAlphaBits = 0
            pfd.cDepthBits      = 16
            pfd.cStencilBits    = 1
            pfd.cAuxBuffers     = 0
            pfd.iLayerType      = PFD_MAIN_PLANE
            pfd.bReserved       = 0
            pfd.dwLayerMask     = 0
            pfd.dwVisibleMask   = 0
            pfd.dwDamageMask    = 0

            if not mhDC := GetDC mhWnd then
                Destroy()
                'error no device context
                MessageBox NULL, "GetDC", "ERROR", MB_OK
                return 0
            end if

            if not mhDC := GetDC mhWnd then
                Destroy()
                'error no device context
                MessageBox NULL, "GetDC", "ERROR", MB_OK
                return 0
            end if

            if not mPixelFormat := ChoosePixelFormat mhDC, @pfd then
                Destroy()
                'error no good pfd
                MessageBox NULL, "ChoosePixelFormat", "ERROR", MB_OK
                return 0
            end if

            if not SetPixelFormat mhDC, mPixelFormat, @pfd then
                Destroy()
                'error could not set
                MessageBox NULL, "SetPixelFormat", "ERROR", MB_OK
                return 0
            end if

            if not mhRC := wglCreateContext mhDC then
                Destroy()
                'error
                MessageBox NULL, "wglCreateContext", "ERROR", MB_OK
                return 0
            end if

            if not wglMakeCurrent mhDC, mhRC  then
                Destroy()
                'error
                MessageBox NULL, "wglMakeCurrent", "ERROR", MB_OK
                return 0
            end if

            'SYNC WITH MONITOR FRAME REFRESH - 60hZ
            sys p=wglgetprocaddress "wglSwapIntervalEXT" : call p 1

            ShowWindow mhWnd, SW_SHOW
            SetForegroundWindow mhWnd
            SetFocus mhWnd

            'setup the program's perspective view
            Resize()
            glShadeModel GL_SMOOTH
            glClearColor 0, 0, 0, 0
            glClearDepth 1
            glEnable GL_DEPTH_TEST
            glDepthFunc GL_LEQUAL
            glHint GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST

            mIsActive  = 1
            mIsRunning = 2 'message loop with callbacks
        end method


        method Reset()
            mIsRunning = 1 'message loop but no callbacks
            if mFullScreen then
                ChangeDisplaySettings NULL, 0
                ShowCursor 1
            end if
            wglMakeCurrent 0, 0
            wglDeleteContext mhRC : mhRC = 0
            ReleaseDC mhWnd, mhDC : mhDC = 0
            PostQuitMessage mhWnd, 0
            DestroyWindow mhWnd : mToDestroy = 0 : mhWnd = 0 : mIsRunning = 0
        end method


        method Destroy()
            Reset()
            UnregisterClass *mClassName, mhInstance
        end method    


        method CreateTTFont( char* aFontName = "Lucida Console", int aSize = 16 ) as sys
            typedef sys HFONT
            HFONT hFont
            sys glFont = glGenLists 256
'hFont  = CreateFont 16,16,0,0,0,0,0,0,0,0,0,0,0,"Lucida Console"

            hFont  = CreateFont     aSize,     'height
                                    aSize,     'width
                                    0,         'escapement
                                    0,         'orientation
                                    FW_NORMAL, 'weight 
                                    0,         'italic
                                    0,         'underline
                                    0,         'strikeout
                                    ANSI_CHARSET, 
                                    OUT_TT_PRECIS, 
                                    CLIP_DEFAULT_PRECIS, 
                                    ANTIALIASED_QUALITY, 
                                    FF_DONTCARE | DEFAULT_PITCH, 
                                    @aFontName

            SelectObject mhDC, hFont
            wglUseFontBitmapsA mhDC, 0, 256, glFont
            return glFont         
        end method


        method DestroyTTFont( sys aFont )
            if aFont then 
                glDeleteLists aFont, 256
            end if
        end method


        method SetTTFont( sys aFont )
            mFont = aFont
        end method


        method GetTTFont() as sys
            return mFont
        end method

        
        method Print( int aX, aY, char* aPrintString )
            if @aPrintString = NULL then return
            glPushMatrix
                glLoadIdentity
                glRasterPos2d aX, aY
                glListBase mFont 
                glCallLists len( aPrintString ), GL_UNSIGNED_BYTE, aPrintString
            glPopMatrix
        end method
        
        
        method BeginTextMode()
            glMatrixMode GL_PROJECTION
            glPushMatrix
                glLoadIdentity
                glOrtho 0, mWidth, 0, mHeight, -1, 1
                glMatrixMode GL_MODELVIEW       
        end method
        
        method EndTextMode()
                glMatrixMode GL_PROJECTION
            glPopMatrix
            glMatrixMode GL_MODELVIEW       
        end method

        method MsgLoop()
            while mIsRunning
                if mIsRunning = 2 then
                    if mRenderFunc then call mRenderFunc
                    if mInputFunc  then call mInputFunc
                    if mToDestroy  then Destroy()
                end if
                mMenuCommand = NULL
                while PeekMessage    @mMSG, NULL, 0, 0, PM_REMOVE
                    TranslateMessage @mMSG
                    DispatchMessage  @mMSG
                wend
            wend
        end method


        method ToggleFullScreen() as sys
            mFullScreen = 1 - mFullScreen
            return mFullScreen
        end method


        method GetWidth() as sys
            return mWidth
        end method


        method GetHeight() as sys
            return mHeight
        end method
        
        
        method KeyEvent() as sys 'read once and clear
            method = mKeydown
            mKeydown = 0
        end method


        method Flip()
            glFlush
            SwapBuffers mhDC
        end method
        

    private

        method MenuDown( WORD ID ) as sys
            if mMenuCommand = ID then return 1
        end method


        method KeyDown( sys ucKey ) as sys
            if mKeys[ucKey] then return 1
        end method

        
        method KeyUp( sys ucKey ) as sys
            if mKeys[ucKey] = 0 then return 1
        end method

        
        method MouseDown( sys button ) as sys
            if mMouseButton and button then return 1
        end method

        
        method MouseUp( sys button ) as sys
            if ( mMouseButton xor -1 ) and button then return 1
        end method


        method Resize()
            if  mHeight = 0 then
                mHeight = 1
            end if
            glViewport 0, 0, mWidth, mHeight

            glMatrixMode GL_PROJECTION
            glLoadIdentity

            double ar = mWidth / mHeight
            gluPerspective mFOV, ar, mNearDistance, mFarDistance

            glMatrixMode GL_MODELVIEW
            glLoadIdentity
            mChangeSize = 0
        end method
    

        method WindowProc( sys mhWnd, uMsg, wParam, lParam )
            select uMsg
                case WM_SYSCOMMAND
                    select wParam
                        case SC_SCREENSAVE
                        case SC_MONITORPOWER
                    end select
                    
                case WM_COMMAND
                    mMenuCommand = LOWORD wParam

                case WM_KEYDOWN
                    mKeys[wParam] = 1
                    mKeydown = wparam
                    return 0

                case WM_KEYUP
                    mKeys[wParam]=0
                    return 0

                case WM_MOUSEMOVE
                    mMouseX = LOWORD lParam
                    mMouseY = HIWORD lParam
                    mMouseButton = wParam
                    return 0

                case WM_MOVE
                    if mIsRunning = 2 then
                        if mRenderFunc then call mRenderFunc
                    end if

                case WM_SIZE
                    mWidth = LOWORD lParam
                    mHeight = HIWORD lParam 
                    mChangeSize = 1
                    resize()
                    if mIsRunning = 2 then
                        if mRenderFunc then call mRenderFunc
                    end if
                    return 0

                case WM_DESTROY
                  mToDestroy = 1
                  return 0
            end select
            return DefWindowProc mhWnd, uMsg, wParam, lParam
        end method
end class

